import './index.css';
import nameGenerator from './name-generator';
import isDef from './is-def';
import mqtt from 'mqtt';
  
/************************************************************************/
/*                        MODELE OBJET                                  */
/***********************************************************************/  
class Sensor {
  constructor(id,values,typeData){
    this.id = id;
    this.values = values;
    this.typeData = typeData;
  }
  getID(){
    return this.id;
  }
  getNbValues(){
    return this.values.length;
  }
  getTypeData(){
    return this.typeData;
  }
  getMoyenneValue(){
    let somme = this.values.reduce(function(a,b){ return a+b; });
    let moyenne = somme / this.values.length;
    return moyenne;
  }
  toString(){
    var s = '<th>' + this.id + '</th> <th>&nbsp;&nbsp;&nbsp;</th>  <th>' + this.values;
    //Cas des valeurs convertible en nombre
    if(!isNaN(Number(this._values))) {
      s+= '</th><th>&nbsp;&nbsp;&nbsp;</th><th>' + this.getMoyenneValue() / this.getNbValues() + '</th>';
    }
    else {
      s+= '</th><th></th><th></th>';
    }
    s+= '<th>&nbsp;&nbsp;&nbsp;</th><th>' + this.typeData + '</th>'; 
    return s;
  }
}


/*************************************************************************/
/************                      FIN                     ***************/
/************************************************************************/

// Store/retrieve the name in/from a cookie.
const cookies = document.cookie.split(';');
console.log(cookies)
let wsname = cookies.find(function(c) {
  if (c.match(/wsname/) !== null) return true;
  return false;
});
if (isDef(wsname)) {
  wsname = wsname.split('=')[1];
} else {
  wsname = nameGenerator();
  document.cookie = "wsname=" + encodeURIComponent(wsname);
}

// Set the name in the header
document.querySelector('header>p').textContent = decodeURIComponent(wsname);



//***************************************************************************************************************************************************************************************

// MQTT Connexion 
var monclient  = mqtt.connect('mqtt://127.0.0.1:8080');
//Subscrubtion to messages à
monclient.on('connect',function(){
  monclient.subscribe('#');
});

//Tableau dans lequel on va répertorier la liste des capteurs
var listeSensors = [];

monclient.on('message', function(topic,message){
  //identifiant du capteur récupéré à partir du topic
  //var val = topic.search('/');//indice du séparateur / dans le topic
  var idCapteur = topic.substring(6);//on extrait l'id du capteur
  console.log(idCapteur);
  //valeur du capteur et type de données
  var json = JSON.parse(message);
  var valeursCapteur = json.value;
  var typeDonnees = json.type;
  //Compteur et Positionnement dans la liste des capteurs que l'on va créer à partir des données reçues
  var i; var pos = -1;
  //Recherche
  for(i = 0; i < listeSensors.length; i++)  {
    if(listeSensors[i].id === idCapteur) {
      pos = i;
    }
  }
  //Ajout d'un capteur
  if(pos === -1){//Là il s'agit d'un nouveau capteur
      var nouveaucapteur;
      nouveaucapteur = new Sensor(idCapteur,valeursCapteur,typeDonnees);
      listeSensors.push(nouveaucapteur);
      var elemMessages = document.getElementById('messages');
      var line = document.createElement('tr');
      //Ajout d'un id à la ligne
      line.id = idCapteur;
      //Creation du contenu de la ligne
      line.innerHTML = nouveaucapteur.toString();
      elemMessages.appendChild(line);  
  }else {//Là il s'agit d'un ancien capteur que l'on va mettre à jour
    //Mise à jour de la valeur du capteur
    listeSensors[pos].values = valeursCapteur;
    //Mise à jour de la page
    document.getElementById(idCapteur).innerHTML = listeSensors[pos].toString(); 
    
  }
});
